searchData={"items":[{"type":"module","title":"JSONAPI","doc":"A module for working with the JSON API specification in Elixir","ref":"JSONAPI.html"},{"type":"function","title":"JSONAPI.json_library/0","doc":"Returns the configured JSON encoding library for JSONAPI.\nTo customize the JSON library, including the following\nin your `config/config.exs`:\n    config :jsonapi, :json_library, Jason","ref":"JSONAPI.html#json_library/0"},{"type":"function","title":"JSONAPI.mime_type/0","doc":"This returns the MIME type for JSONAPIs","ref":"JSONAPI.html#mime_type/0"},{"type":"module","title":"JSONAPI.Config","doc":"Configuration struct containing JSON API information for a request","ref":"JSONAPI.Config.html"},{"type":"type","title":"JSONAPI.Config.t/0","doc":"","ref":"JSONAPI.Config.html#t:t/0"},{"type":"module","title":"JSONAPI.ContentTypeNegotiation","doc":"Provides content type negotiation by validating the `content-type`\nand `accept` headers.\n\nThe proper jsonapi.org content type is\n`application/vnd.api+json`. As per [the spec](http://jsonapi.org/format/#content-negotiation-servers)\n\nThis plug does three things:\n\n1. Returns 415 unless the content-type header is correct.\n2. Returns 406 unless the accept header is correct.\n3. Registers a before send hook to set the content-type if not already set.","ref":"JSONAPI.ContentTypeNegotiation.html"},{"type":"function","title":"JSONAPI.ContentTypeNegotiation.call/2","doc":"","ref":"JSONAPI.ContentTypeNegotiation.html#call/2"},{"type":"function","title":"JSONAPI.ContentTypeNegotiation.init/1","doc":"","ref":"JSONAPI.ContentTypeNegotiation.html#init/1"},{"type":"module","title":"JSONAPI.Deprecation","doc":"Generate warnings in places where we want to deprecate functions or struct parameters","ref":"JSONAPI.Deprecation.html"},{"type":"function","title":"JSONAPI.Deprecation.warn/1","doc":"Generates a deprecation warning for using `fields[relationship_key]` instead of `fields[type]` when\n  parsing query parameters.","ref":"JSONAPI.Deprecation.html#warn/1"},{"type":"module","title":"JSONAPI.Deserializer","doc":"This plug flattens incoming params for ease of use when casting to changesets.\nAs a result, you are able to pattern match specific attributes in your controller\nactions.\n\nNote that this Plug will only deserialize your payload when the request's content\ntype is for a JSON:API request (i.e. \"application/vnd.api+json\"). All other\ncontent types will be ignored.","ref":"JSONAPI.Deserializer.html"},{"type":"module","title":"Example - JSONAPI.Deserializer","doc":"For example these params:\n    %{\n      \"data\" => %{\n        \"id\" => \"1\",\n        \"type\" => \"user\",\n        \"attributes\" => %{\n          \"foo-bar\" => true\n        },\n        \"relationships\" => %{\n          \"baz\" => %{\"data\" => %{\"id\" => \"2\", \"type\" => \"baz\"}}\n        }\n      }\n    }\n\nare transformed to:\n\n    %{\n      \"id\" => \"1\",\n      \"type\" => \"user\"\n      \"foo-bar\" => true,\n      \"baz-id\" => \"2\"\n    }","ref":"JSONAPI.Deserializer.html#module-example"},{"type":"module","title":"Usage - JSONAPI.Deserializer","doc":"Just include in your plug stack _after_ a json parser:\n    plug Plug.Parsers, parsers: [:json], json_decoder: Jason\n    plug JSONAPI.Deserializer\n\nor a part of your Controller plug pipeline\n    plug JSONAPI.Deserializer\n\nIn addition, if you want to underscore your parameters\n    plug JSONAPI.Deserializer\n    plug JSONAPI.UnderscoreParameters","ref":"JSONAPI.Deserializer.html#module-usage"},{"type":"function","title":"JSONAPI.Deserializer.call/2","doc":"","ref":"JSONAPI.Deserializer.html#call/2"},{"type":"function","title":"JSONAPI.Deserializer.init/1","doc":"","ref":"JSONAPI.Deserializer.html#init/1"},{"type":"module","title":"JSONAPI.EnsureSpec","doc":"A helper Plug to enforce the JSON API specification","ref":"JSONAPI.EnsureSpec.html"},{"type":"function","title":"JSONAPI.EnsureSpec.call/2","doc":"Callback implementation for  Plug.call/2 .","ref":"JSONAPI.EnsureSpec.html#call/2"},{"type":"function","title":"JSONAPI.EnsureSpec.init/1","doc":"Callback implementation for  Plug.init/1 .","ref":"JSONAPI.EnsureSpec.html#init/1"},{"type":"module","title":"JSONAPI.ErrorView","doc":"","ref":"JSONAPI.ErrorView.html"},{"type":"function","title":"JSONAPI.ErrorView.build_error/5","doc":"","ref":"JSONAPI.ErrorView.html#build_error/5"},{"type":"type","title":"JSONAPI.ErrorView.error_attrs/0","doc":"","ref":"JSONAPI.ErrorView.html#t:error_attrs/0"},{"type":"function","title":"JSONAPI.ErrorView.incorrect_content_type/0","doc":"","ref":"JSONAPI.ErrorView.html#incorrect_content_type/0"},{"type":"function","title":"JSONAPI.ErrorView.malformed_id/0","doc":"","ref":"JSONAPI.ErrorView.html#malformed_id/0"},{"type":"function","title":"JSONAPI.ErrorView.mismatched_id/0","doc":"","ref":"JSONAPI.ErrorView.html#mismatched_id/0"},{"type":"function","title":"JSONAPI.ErrorView.missing_data_attributes_param/0","doc":"","ref":"JSONAPI.ErrorView.html#missing_data_attributes_param/0"},{"type":"function","title":"JSONAPI.ErrorView.missing_data_id_param/0","doc":"","ref":"JSONAPI.ErrorView.html#missing_data_id_param/0"},{"type":"function","title":"JSONAPI.ErrorView.missing_data_param/0","doc":"","ref":"JSONAPI.ErrorView.html#missing_data_param/0"},{"type":"function","title":"JSONAPI.ErrorView.missing_data_type_param/0","doc":"","ref":"JSONAPI.ErrorView.html#missing_data_type_param/0"},{"type":"function","title":"JSONAPI.ErrorView.missing_id/0","doc":"","ref":"JSONAPI.ErrorView.html#missing_id/0"},{"type":"function","title":"JSONAPI.ErrorView.missing_relationship_data_id_param_error_attrs/1","doc":"","ref":"JSONAPI.ErrorView.html#missing_relationship_data_id_param_error_attrs/1"},{"type":"function","title":"JSONAPI.ErrorView.missing_relationship_data_param_error_attrs/1","doc":"","ref":"JSONAPI.ErrorView.html#missing_relationship_data_param_error_attrs/1"},{"type":"function","title":"JSONAPI.ErrorView.missing_relationship_data_type_param_error_attrs/1","doc":"","ref":"JSONAPI.ErrorView.html#missing_relationship_data_type_param_error_attrs/1"},{"type":"function","title":"JSONAPI.ErrorView.relationships_missing_object/0","doc":"","ref":"JSONAPI.ErrorView.html#relationships_missing_object/0"},{"type":"function","title":"JSONAPI.ErrorView.send_error/2","doc":"","ref":"JSONAPI.ErrorView.html#send_error/2"},{"type":"function","title":"JSONAPI.ErrorView.send_error/3","doc":"","ref":"JSONAPI.ErrorView.html#send_error/3"},{"type":"function","title":"JSONAPI.ErrorView.serialize_error/1","doc":"","ref":"JSONAPI.ErrorView.html#serialize_error/1"},{"type":"function","title":"JSONAPI.ErrorView.serialize_errors/1","doc":"","ref":"JSONAPI.ErrorView.html#serialize_errors/1"},{"type":"function","title":"JSONAPI.ErrorView.to_many_relationships_payload_for_standard_endpoint/0","doc":"","ref":"JSONAPI.ErrorView.html#to_many_relationships_payload_for_standard_endpoint/0"},{"type":"module","title":"JSONAPI.Exceptions","doc":"","ref":"JSONAPI.Exceptions.html"},{"type":"module","title":"JSONAPI.FormatRequired","doc":"Enforces the JSONAPI format of {\"data\" => {\"attributes\" => ...}} for request bodies","ref":"JSONAPI.FormatRequired.html"},{"type":"function","title":"JSONAPI.FormatRequired.call/2","doc":"","ref":"JSONAPI.FormatRequired.html#call/2"},{"type":"function","title":"JSONAPI.FormatRequired.init/1","doc":"","ref":"JSONAPI.FormatRequired.html#init/1"},{"type":"module","title":"JSONAPI.IdRequired","doc":"Ensure that the URL id matches the id in the request body and is a string","ref":"JSONAPI.IdRequired.html"},{"type":"function","title":"JSONAPI.IdRequired.call/2","doc":"","ref":"JSONAPI.IdRequired.html#call/2"},{"type":"function","title":"JSONAPI.IdRequired.init/1","doc":"","ref":"JSONAPI.IdRequired.html#init/1"},{"type":"behaviour","title":"JSONAPI.Paginator","doc":"Pagination strategy behaviour","ref":"JSONAPI.Paginator.html"},{"type":"type","title":"JSONAPI.Paginator.links/0","doc":"","ref":"JSONAPI.Paginator.html#t:links/0"},{"type":"type","title":"JSONAPI.Paginator.options/0","doc":"","ref":"JSONAPI.Paginator.html#t:options/0"},{"type":"type","title":"JSONAPI.Paginator.page/0","doc":"","ref":"JSONAPI.Paginator.html#t:page/0"},{"type":"callback","title":"JSONAPI.Paginator.paginate/5","doc":"","ref":"JSONAPI.Paginator.html#c:paginate/5"},{"type":"type","title":"JSONAPI.Paginator.params/0","doc":"","ref":"JSONAPI.Paginator.html#t:params/0"},{"type":"module","title":"JSONAPI.QueryParser","doc":"Implements a fully JSONAPI V1 spec for parsing a complex query string via the\n`query_params` field from a `Plug.Conn` struct and returning Elixir datastructures.\nThe purpose is to validate and encode incoming queries and fail quickly.\n\nPrimarialy this handles:\n  * [sorts](http://jsonapi.org/format/#fetching-sorting)\n  * [include](http://jsonapi.org/format/#fetching-includes)\n  * [filtering](http://jsonapi.org/format/#fetching-filtering)\n  * [sparse fieldsets](https://jsonapi.org/format/#fetching-sparse-fieldsets)\n  * [pagination](http://jsonapi.org/format/#fetching-pagination)\n\nThis Plug works in conjunction with a `JSONAPI.View` as well as some Plug\ndefined configuration.\n\nIn your controller you may add:\n\n```\nplug JSONAPI.QueryParser,\n  filter: ~w(title),\n  sort: ~w(created_at title),\n  include: ~w(others) # optionally specify a list of allowed includes.\n  view: MyView\n```\n\nIf you specify which includes are allowed, any include name not in the list\nwill produce an error. If you omit the `include` list then all relationships\nspecified by the given resource will be allowed.\n\nIf your controller's index function receives a query with params inside those\nbounds it will build a `JSONAPI.Config` that has all the validated and parsed\nfields for your usage. The final configuration will be added to assigns\n`jsonapi_query`.\n\nThe final output will be a `JSONAPI.Config` struct and will look similar to the\nfollowing:\n\n    %JSONAPI.Config{\n      view: MyView,\n      opts: [view: MyView, sort: [\"created_at\", \"title\"], filter: [\"title\"]],\n      sort: [desc: :created_at] # Easily insertable into an ecto order_by,\n      filter: [title: \"my title\"] # Easily reduceable into ecto where clauses\n      include: [comments: :user] # Easily insertable into a Repo.preload,\n      fields: %{\"myview\" => [:id, :text], \"comment\" => [:id, :body],\n      page: %{\n        limit: limit,\n        offset: offset,\n        page: page,\n        size: size,\n        cursor: cursor\n      }}\n    }\n\nThe final result should allow you to build a query quickly and with little overhead.","ref":"JSONAPI.QueryParser.html"},{"type":"module","title":"Sparse Fieldsets - JSONAPI.QueryParser","doc":"Sparse fieldsets are supported. By default your response will include all\navailable fields. Note that the query to your database is left to you. Should\nyou want to query your DB for specific fields `JSONAPI.Config.fields` will\nreturn the requested fields for each resource (see above example).","ref":"JSONAPI.QueryParser.html#module-sparse-fieldsets"},{"type":"module","title":"Options - JSONAPI.QueryParser","doc":"* `:view` - The JSONAPI View which is the basis for this plug.\n  * `:sort` - List of atoms which define which fields can be sorted on.\n  * `:filter` - List of atoms which define which fields can be filtered on.","ref":"JSONAPI.QueryParser.html#module-options"},{"type":"module","title":"Dasherized Fields - JSONAPI.QueryParser","doc":"Note that if your API is returning dasherized fields (e.g. `\"dog-breed\": \"Corgi\"`)\nwe recommend that you include the `JSONAPI.UnderscoreParameters` Plug in your\nAPI's pipeline with the `replace_query_params` option set to `true`. This will\nunderscore fields for easier operations in your code.\n\nFor more details please see `JSONAPI.UnderscoreParameters`.","ref":"JSONAPI.QueryParser.html#module-dasherized-fields"},{"type":"function","title":"JSONAPI.QueryParser.build_sort/2","doc":"","ref":"JSONAPI.QueryParser.html#build_sort/2"},{"type":"function","title":"JSONAPI.QueryParser.get_valid_fields_for_type/2","doc":"","ref":"JSONAPI.QueryParser.html#get_valid_fields_for_type/2"},{"type":"function","title":"JSONAPI.QueryParser.get_view_for_type/2","doc":"","ref":"JSONAPI.QueryParser.html#get_view_for_type/2"},{"type":"function","title":"JSONAPI.QueryParser.handle_include/2","doc":"","ref":"JSONAPI.QueryParser.html#handle_include/2"},{"type":"function","title":"JSONAPI.QueryParser.handle_nested_include/3","doc":"","ref":"JSONAPI.QueryParser.html#handle_nested_include/3"},{"type":"function","title":"JSONAPI.QueryParser.parse_fields/2","doc":"","ref":"JSONAPI.QueryParser.html#parse_fields/2"},{"type":"function","title":"JSONAPI.QueryParser.parse_filter/2","doc":"","ref":"JSONAPI.QueryParser.html#parse_filter/2"},{"type":"function","title":"JSONAPI.QueryParser.parse_include/2","doc":"","ref":"JSONAPI.QueryParser.html#parse_include/2"},{"type":"function","title":"JSONAPI.QueryParser.parse_pagination/2","doc":"","ref":"JSONAPI.QueryParser.html#parse_pagination/2"},{"type":"function","title":"JSONAPI.QueryParser.parse_sort/2","doc":"","ref":"JSONAPI.QueryParser.html#parse_sort/2"},{"type":"module","title":"JSONAPI.ResponseContentType","doc":"Simply add this plug to your endpoint or your router :api pipeline and it will\nensure you return the correct response type.\n\nIf you need to override the response type simple set conn.assigns[:override_jsonapi]\nand this will be skipped.","ref":"JSONAPI.ResponseContentType.html"},{"type":"function","title":"JSONAPI.ResponseContentType.call/2","doc":"Callback implementation for  Plug.call/2 .","ref":"JSONAPI.ResponseContentType.html#call/2"},{"type":"function","title":"JSONAPI.ResponseContentType.init/1","doc":"Callback implementation for  Plug.init/1 .","ref":"JSONAPI.ResponseContentType.html#init/1"},{"type":"module","title":"JSONAPI.Serializer","doc":"Serialize a map of data into a properly formatted JSON API response object","ref":"JSONAPI.Serializer.html"},{"type":"function","title":"JSONAPI.Serializer.build_relationships/5","doc":"","ref":"JSONAPI.Serializer.html#build_relationships/5"},{"type":"function","title":"JSONAPI.Serializer.build_relationships/7","doc":"","ref":"JSONAPI.Serializer.html#build_relationships/7"},{"type":"function","title":"JSONAPI.Serializer.data_loaded?/1","doc":"","ref":"JSONAPI.Serializer.html#data_loaded?/1"},{"type":"type","title":"JSONAPI.Serializer.document/0","doc":"","ref":"JSONAPI.Serializer.html#t:document/0"},{"type":"function","title":"JSONAPI.Serializer.encode_data/5","doc":"","ref":"JSONAPI.Serializer.html#encode_data/5"},{"type":"function","title":"JSONAPI.Serializer.encode_rel_data/2","doc":"","ref":"JSONAPI.Serializer.html#encode_rel_data/2"},{"type":"function","title":"JSONAPI.Serializer.encode_relation/1","doc":"","ref":"JSONAPI.Serializer.html#encode_relation/1"},{"type":"function","title":"JSONAPI.Serializer.encode_relationships/4","doc":"","ref":"JSONAPI.Serializer.html#encode_relationships/4"},{"type":"function","title":"JSONAPI.Serializer.extrapolate_relationship_config/1","doc":"Given the relationship config entry provided by a JSONAPI.View, produce\nthe extrapolated config tuple containing:\n  - The name of the relationship to be used when serializing\n  - The key in the data the relationship is found under\n  - The relationship resource's JSONAPI.View module\n  - A boolean for whether the relationship is included by default or not","ref":"JSONAPI.Serializer.html#extrapolate_relationship_config/1"},{"type":"function","title":"JSONAPI.Serializer.flatten_included/1","doc":"","ref":"JSONAPI.Serializer.html#flatten_included/1"},{"type":"function","title":"JSONAPI.Serializer.serialize/5","doc":"Takes a view, data and a optional plug connection and returns a fully JSONAPI Serialized document.\nThis assumes you are using the JSONAPI.View and have data in maps or structs.\n\nPlease refer to `JSONAPI.View` for more information. If you are in interested in relationships\nand includes you may also want to reference the `JSONAPI.QueryParser`.","ref":"JSONAPI.Serializer.html#serialize/5"},{"type":"module","title":"JSONAPI.UnderscoreParameters","doc":"Takes dasherized JSON:API params and converts them to underscored params. Add\nthis to your API's pipeline to aid in dealing with incoming parameters such as query\nparams or data.\n\nBy default the newly underscored params will only replace the existing `params` field\nof the `Plug.Conn` struct, but leave the `query_params` and `body_params` untouched.\nIf you are using the `JSONAPI.QueryParser` and need to also have the `query_params` on\nthe `Plug.Conn` updated, set the `replace_query_params` option to `true`.\n\nNote that this Plug will only underscore parameters when the request's content\ntype is for a JSON:API request (i.e. \"application/vnd.api+json\"). All other\ncontent types will be ignored.","ref":"JSONAPI.UnderscoreParameters.html"},{"type":"module","title":"Options - JSONAPI.UnderscoreParameters","doc":"* `:replace_query_params` - When `true`, it will replace the `query_params` field in\n  the `Plug.Conn` struct.  This is useful when you have downstream code which is\n  expecting underscored fields in `query_params`, and not just in `params`. Defaults\n  to `false`.","ref":"JSONAPI.UnderscoreParameters.html#module-options"},{"type":"module","title":"Example - JSONAPI.UnderscoreParameters","doc":"%{\n  \"data\" => %{\n    \"attributes\" => %{\n      \"foo-bar\" => true\n    }\n  }\n}\n\nare transformed to:\n\n%{\n  \"data\" => %{\n    \"attributes\" => %{\n      \"foo_bar\" => true\n    }\n  }\n}\n\nMoreover, with a GET request like:\n\n    GET /example?filters[dog-breed]=Corgi\n\n**Without** this Plug your index action would look like:\n\n    def index(conn, %{\"filters\" => %{\"dog-breed\" => \"Corgi\"}})\n\nAnd **with** this Plug:\n\n    def index(conn, %{\"filters\" => %{\"dog_breed\" => \"Corgi\"}})\n\nYour API's pipeline might look something like this:\n\n    # e.g. a Phoenix app\n\n    pipeline :api do\n      plug JSONAPI.EnforceSpec\n      plug JSONAPI.UnderscoreParameters\n    end","ref":"JSONAPI.UnderscoreParameters.html#module-example"},{"type":"module","title":"JSONAPI.Utils.DataToParams","doc":"Converts a Map representation of the JSON:API resource object format into a flat Map convenient for\nchangeset casting.","ref":"JSONAPI.Utils.DataToParams.html"},{"type":"function","title":"JSONAPI.Utils.DataToParams.process/1","doc":"","ref":"JSONAPI.Utils.DataToParams.html#process/1"},{"type":"module","title":"JSONAPI.Utils.IncludeTree","doc":"Internal utility for building trees of resource relationships","ref":"JSONAPI.Utils.IncludeTree.html"},{"type":"function","title":"JSONAPI.Utils.IncludeTree.build_tree/2","doc":"","ref":"JSONAPI.Utils.IncludeTree.html#build_tree/2"},{"type":"function","title":"JSONAPI.Utils.IncludeTree.deep_merge/2","doc":"","ref":"JSONAPI.Utils.IncludeTree.html#deep_merge/2"},{"type":"function","title":"JSONAPI.Utils.IncludeTree.get_base_relationships/1","doc":"","ref":"JSONAPI.Utils.IncludeTree.html#get_base_relationships/1"},{"type":"function","title":"JSONAPI.Utils.IncludeTree.member_of_tree?/2","doc":"","ref":"JSONAPI.Utils.IncludeTree.html#member_of_tree?/2"},{"type":"function","title":"JSONAPI.Utils.IncludeTree.put_as_tree/3","doc":"","ref":"JSONAPI.Utils.IncludeTree.html#put_as_tree/3"},{"type":"module","title":"JSONAPI.Utils.String","doc":"String manipulation helpers.","ref":"JSONAPI.Utils.String.html"},{"type":"function","title":"JSONAPI.Utils.String.camelize/1","doc":"Replace underscores or dashes between words in `value` with camelCasing\n\nIgnores underscores or dashes that are not between letters/numbers","ref":"JSONAPI.Utils.String.html#camelize/1"},{"type":"function","title":"Examples - JSONAPI.Utils.String.camelize/1","doc":"iex> camelize(\"top_posts\")\n    \"topPosts\"\n\n    iex> camelize(:top_posts)\n    \"topPosts\"\n\n    iex> camelize(\"_top_posts\")\n    \"_topPosts\"\n\n    iex> camelize(\"_top__posts_\")\n    \"_top__posts_\"\n\n    iex> camelize(\"\")\n    \"\"\n\n    iex> camelize(\"alreadyCamelized\")\n    \"alreadyCamelized\"\n\n    iex> camelize(\"alreadyCamelized-id\")\n    \"alreadyCamelizedId\"\n\n    iex> camelize(\"alreadyCamelized_id\")\n    \"alreadyCamelizedId\"\n\n    iex> camelize(\"PascalLambda_id\")\n    \"pascalLambdaId\"","ref":"JSONAPI.Utils.String.html#camelize/1-examples"},{"type":"function","title":"JSONAPI.Utils.String.dasherize/1","doc":"Replace underscores between words in `value` with dashes\n\nIgnores underscores that are not between letters/numbers","ref":"JSONAPI.Utils.String.html#dasherize/1"},{"type":"function","title":"Examples - JSONAPI.Utils.String.dasherize/1","doc":"iex> dasherize(\"top_posts\")\n    \"top-posts\"\n\n    iex> dasherize(\"_top_posts\")\n    \"_top-posts\"\n\n    iex> dasherize(\"_top__posts_\")\n    \"_top__posts_\"\n\n    iex> dasherize(\"ages_0_17\")\n    \"ages-0-17\"","ref":"JSONAPI.Utils.String.html#dasherize/1-examples"},{"type":"function","title":"JSONAPI.Utils.String.expand_fields/2","doc":"","ref":"JSONAPI.Utils.String.html#expand_fields/2"},{"type":"function","title":"Examples - JSONAPI.Utils.String.expand_fields/2","doc":"iex> expand_fields(%{\"foo-bar\" => \"baz\"}, &underscore/1)\n    %{\"foo_bar\" => \"baz\"}\n\n    iex> expand_fields(%{\"foo_bar\" => \"baz\"}, &dasherize/1)\n    %{\"foo-bar\" => \"baz\"}\n\n    iex> expand_fields(%{\"foo-bar\" => \"baz\"}, &camelize/1)\n    %{\"fooBar\" => \"baz\"}\n\n    iex> expand_fields({\"foo-bar\", \"dollar-sol\"}, &underscore/1)\n    {\"foo_bar\", \"dollar-sol\"}\n\n    iex> expand_fields({\"foo-bar\", %{\"a-d\" => \"z-8\"}}, &underscore/1)\n    {\"foo_bar\", %{\"a_d\" => \"z-8\"}}\n\n    iex> expand_fields(%{\"f-b\" => %{\"a-d\" => \"z\"}, \"c-d\" => \"e\"}, &underscore/1)\n    %{\"f_b\" => %{\"a_d\" => \"z\"}, \"c_d\" => \"e\"}\n\n    iex> expand_fields(%{\"f-b\" => %{\"a-d\" => %{\"z-w\" => \"z\"}}, \"c-d\" => \"e\"}, &underscore/1)\n    %{\"f_b\" => %{\"a_d\" => %{\"z_w\" => \"z\"}}, \"c_d\" => \"e\"}\n\n    iex> expand_fields(:\"foo-bar\", &underscore/1)\n    \"foo_bar\"\n\n    iex> expand_fields(:foo_bar, &dasherize/1)\n    \"foo-bar\"\n\n    iex> expand_fields(:\"foo-bar\", &camelize/1)\n    \"fooBar\"\n\n    iex> expand_fields(%{\"f-b\" => \"a-d\"}, &underscore/1)\n    %{\"f_b\" => \"a-d\"}\n\n    iex> expand_fields(%{\"inserted-at\" => ~N[2019-01-17 03:27:24.776957]}, &underscore/1)\n    %{\"inserted_at\" => ~N[2019-01-17 03:27:24.776957]}\n\n    iex> expand_fields(%{\"xValue\" => 123}, &underscore/1)\n    %{\"x_value\" => 123}\n\n    iex> expand_fields(%{\"attributes\" => %{\"corgiName\" => \"Wardel\"}}, &underscore/1)\n    %{\"attributes\" => %{\"corgi_name\" => \"Wardel\"}}\n\n    iex> expand_fields(%{\"attributes\" => %{\"corgiName\" => [\"Wardel\"]}}, &underscore/1)\n    %{\"attributes\" => %{\"corgi_name\" => [\"Wardel\"]}}\n\n    iex> expand_fields(%{\"attributes\" => %{\"someField\" => [\"SomeValue\", %{\"nestedField\" => \"Value\"}]}}, &underscore/1)\n    %{\"attributes\" => %{\"some_field\" => [\"SomeValue\", %{\"nested_field\" => \"Value\"}]}}\n\n    iex> expand_fields([%{\"fooBar\" => \"a\"}, %{\"fooBar\" => \"b\"}], &underscore/1)\n    [%{\"foo_bar\" => \"a\"}, %{\"foo_bar\" => \"b\"}]\n\n    iex> expand_fields([%{\"foo_bar\" => \"a\"}, %{\"foo_bar\" => \"b\"}], &camelize/1)\n    [%{\"fooBar\" => \"a\"}, %{\"fooBar\" => \"b\"}]\n\n    iex> expand_fields(%{\"fooAttributes\" => [%{\"fooBar\" => \"a\"}, %{\"fooBar\" => \"b\"}]}, &underscore/1)\n    %{\"foo_attributes\" => [%{\"foo_bar\" => \"a\"}, %{\"foo_bar\" => \"b\"}]}\n\n    iex> expand_fields(%{\"foo_attributes\" => [%{\"foo_bar\" => \"a\"}, %{\"foo_bar\" => \"b\"}]}, &camelize/1)\n    %{\"fooAttributes\" => [%{\"fooBar\" => \"a\"}, %{\"fooBar\" => \"b\"}]}\n\n    iex> expand_fields(%{\"foo_attributes\" => [%{\"foo_bar\" => [1, 2]}]}, &camelize/1)\n    %{\"fooAttributes\" => [%{\"fooBar\" => [1, 2]}]}","ref":"JSONAPI.Utils.String.html#expand_fields/2-examples"},{"type":"function","title":"JSONAPI.Utils.String.expand_root_keys/2","doc":"Like `JSONAPI.Utils.String.expand_fields/2`, but only uses the given function to transform the\nkeys of a top-level map. Other values are transformed with `to_string/1`.","ref":"JSONAPI.Utils.String.html#expand_root_keys/2"},{"type":"function","title":"Examples - JSONAPI.Utils.String.expand_root_keys/2","doc":"iex> expand_root_keys(%{\"foo-bar\" => %{\"bar-baz\" => \"x\"}}, &underscore/1)\n    %{\"foo_bar\" => %{\"bar-baz\" => \"x\"}}\n\n    iex> expand_root_keys(%{\"foo-bar\" => [:x, %{\"bar-baz\" => \"y\"}]}, &underscore/1)\n    %{\"foo_bar\" => [\"x\", %{\"bar-baz\" => \"y\"}]}","ref":"JSONAPI.Utils.String.html#expand_root_keys/2-examples"},{"type":"function","title":"JSONAPI.Utils.String.field_transformation/0","doc":"The configured transformation for the API's fields. JSON:API v1.1 recommends\nusing camlized fields (e.g. \"goodDog\", versus \"good_dog\").  However, we don't hold a strong\nopinion, so feel free to customize it how you would like (e.g. \"good-dog\", versus \"good_dog\").\n\nThis library currently supports camelized, dashed and underscored fields. Shallow variants\nexist that only transform top-level field keys.","ref":"JSONAPI.Utils.String.html#field_transformation/0"},{"type":"function","title":"Configuration examples - JSONAPI.Utils.String.field_transformation/0","doc":"camelCase fields:\n\n```\nconfig :jsonapi, field_transformation: :camelize\n```\n\n```\nconfig :jsonapi, field_transformation: :camelize_shallow\n```\n\nDashed fields:\n\n```\nconfig :jsonapi, field_transformation: :dasherize\n```\n\n```\nconfig :jsonapi, field_transformation: :dasherize_shallow\n```\n\nUnderscored fields:\n\n```\nconfig :jsonapi, field_transformation: :underscore\n```","ref":"JSONAPI.Utils.String.html#field_transformation/0-configuration-examples"},{"type":"function","title":"JSONAPI.Utils.String.underscore/1","doc":"Replace dashes between words in `value` with underscores\n\nIgnores dashes that are not between letters/numbers","ref":"JSONAPI.Utils.String.html#underscore/1"},{"type":"function","title":"Examples - JSONAPI.Utils.String.underscore/1","doc":"iex> underscore(\"top-posts\")\n    \"top_posts\"\n\n    iex> underscore(:top_posts)\n    \"top_posts\"\n\n    iex> underscore(\"-top-posts\")\n    \"-top_posts\"\n\n    iex> underscore(\"-top--posts-\")\n    \"-top--posts-\"\n\n    iex> underscore(\"corgiAge\")\n    \"corgi_age\"\n\n    iex> underscore(\"ages-0-17\")\n    \"ages_0_17\"","ref":"JSONAPI.Utils.String.html#underscore/1-examples"},{"type":"behaviour","title":"JSONAPI.View","doc":"A View is simply a module that defines certain callbacks to configure proper\nrendering of your JSONAPI documents.\n\n    defmodule PostView do\n      use JSONAPI.View\n\n      def fields, do: [:id, :text, :body]\n      def type, do: \"post\"\n      def relationships do\n        [author: UserView,\n         comments: CommentView]\n      end\n    end\n\n    defmodule UserView do\n      use JSONAPI.View\n\n      def fields, do: [:id, :username]\n      def type, do: \"user\"\n      def relationships, do: []\n    end\n\n    defmodule CommentView do\n      use JSONAPI.View\n\n      def fields, do: [:id, :text]\n      def type, do: \"comment\"\n      def relationships do\n        [user: {UserView, :include}]\n      end\n    end\n\n    defmodule DogView do\n      use JSONAPI.View, namespace: \"/pupperz-api\"\n    end\n\nYou can now call `UserView.show(user, conn, conn.params)` and it will render\na valid jsonapi doc.","ref":"JSONAPI.View.html"},{"type":"behaviour","title":"Fields - JSONAPI.View","doc":"By default, the resulting JSON document consists of fields, defined in the `fields/0`\nfunction. You can define custom fields or override current fields by defining a\n2-arity function inside the view that takes `data` and `conn` as arguments and has\nthe same name as the field it will be producing. Refer to our `fullname/2` example below.\n\n    defmodule UserView do\n      use JSONAPI.View\n\n      def fullname(data, conn), do: \"fullname\"\n\n      def fields, do: [:id, :username, :fullname]\n      def type, do: \"user\"\n      def relationships, do: []\n    end\n\nFields may be omitted manually using the `hidden/1` function.\n\n    defmodule UserView do\n      use JSONAPI.View\n\n      def fields, do: [:id, :username, :email]\n\n      def type, do: \"user\"\n\n      def hidden(_data) do\n        [:email] # will be removed from the response\n      end\n    end\n\nIn order to use [sparse fieldsets](https://jsonapi.org/format/#fetching-sparse-fieldsets)\nyou must include the `JSONAPI.QueryParser` plug.\n\nIf you want to fetch fields from the given data *dynamically*, you can use the\n`c:get_field/3` callback.\n\n    defmodule UserView do\n      use JSONAPI.View\n\n      def fields, do: [:id, :username, :email]\n\n      def type, do: \"user\"\n\n      def get_field(field, data, _conn) do\n        Map.fetch!(data, field)\n      end\n    end","ref":"JSONAPI.View.html#module-fields"},{"type":"behaviour","title":"Relationships - JSONAPI.View","doc":"Currently the relationships callback expects that a map is returned\nconfiguring the information you will need. If you have the following Ecto\nModel setup\n\n    defmodule User do\n      schema \"users\" do\n        field :username\n        has_many :posts\n        has_one :image\n      end\n    end\n\nand the includes setup from above. If your Post has loaded the author and the\nquery asks for it then it will be loaded.\n\nSo for example:\n`GET /posts?include=post.author` if the author record is loaded on the Post, and you are using\nthe `JSONAPI.QueryParser` it will be included in the `includes` section of the JSONAPI document.\n\nIf you always want to include a relationship. First make sure its always preloaded\nand then use the `[user: {UserView, :include}]` syntax in your `includes` function. This tells\nthe serializer to *always* include if its loaded.","ref":"JSONAPI.View.html#module-relationships"},{"type":"behaviour","title":"Options - JSONAPI.View","doc":"* `:host` (binary) - Allows the `host` to be overridden for generated URLs. Defaults to `host` of the supplied `conn`.\n\n  * `:scheme` (atom) - Enables configuration of the HTTP scheme for generated URLS.  Defaults to `scheme` from the provided `conn`.\n\n  * `:namespace` (binary) - Allows the namespace of a given resource. This may be\n    configured globally or overridden on the View itself. Note that if you have\n    a globally defined namespace and need to *remove* the namespace for a\n    resource, set the namespace to a blank String.\n\nThe default behaviour for `host` and `scheme` is to derive it from the `conn` provided, while the\ndefault style for presentation in names is to be underscored and not dashed.","ref":"JSONAPI.View.html#module-options"},{"type":"callback","title":"JSONAPI.View.attributes/2","doc":"","ref":"JSONAPI.View.html#c:attributes/2"},{"type":"type","title":"JSONAPI.View.data/0","doc":"","ref":"JSONAPI.View.html#t:data/0"},{"type":"type","title":"JSONAPI.View.field/0","doc":"","ref":"JSONAPI.View.html#t:field/0"},{"type":"callback","title":"JSONAPI.View.fields/0","doc":"","ref":"JSONAPI.View.html#c:fields/0"},{"type":"callback","title":"JSONAPI.View.get_field/3","doc":"","ref":"JSONAPI.View.html#c:get_field/3"},{"type":"callback","title":"JSONAPI.View.hidden/1","doc":"","ref":"JSONAPI.View.html#c:hidden/1"},{"type":"callback","title":"JSONAPI.View.id/1","doc":"","ref":"JSONAPI.View.html#c:id/1"},{"type":"type","title":"JSONAPI.View.link_object/0","doc":"","ref":"JSONAPI.View.html#t:link_object/0"},{"type":"type","title":"JSONAPI.View.links/0","doc":"","ref":"JSONAPI.View.html#t:links/0"},{"type":"callback","title":"JSONAPI.View.links/2","doc":"","ref":"JSONAPI.View.html#c:links/2"},{"type":"type","title":"JSONAPI.View.meta/0","doc":"","ref":"JSONAPI.View.html#t:meta/0"},{"type":"callback","title":"JSONAPI.View.meta/2","doc":"","ref":"JSONAPI.View.html#c:meta/2"},{"type":"callback","title":"JSONAPI.View.namespace/0","doc":"","ref":"JSONAPI.View.html#c:namespace/0"},{"type":"type","title":"JSONAPI.View.options/0","doc":"","ref":"JSONAPI.View.html#t:options/0"},{"type":"callback","title":"JSONAPI.View.pagination_links/4","doc":"","ref":"JSONAPI.View.html#c:pagination_links/4"},{"type":"callback","title":"JSONAPI.View.path/0","doc":"","ref":"JSONAPI.View.html#c:path/0"},{"type":"callback","title":"JSONAPI.View.relationships/0","doc":"","ref":"JSONAPI.View.html#c:relationships/0"},{"type":"type","title":"JSONAPI.View.resource_id/0","doc":"","ref":"JSONAPI.View.html#t:resource_id/0"},{"type":"type","title":"JSONAPI.View.resource_type/0","doc":"","ref":"JSONAPI.View.html#t:resource_type/0"},{"type":"type","title":"JSONAPI.View.t/0","doc":"","ref":"JSONAPI.View.html#t:t/0"},{"type":"callback","title":"JSONAPI.View.type/0","doc":"","ref":"JSONAPI.View.html#c:type/0"},{"type":"callback","title":"JSONAPI.View.url_for/2","doc":"","ref":"JSONAPI.View.html#c:url_for/2"},{"type":"function","title":"JSONAPI.View.url_for/3","doc":"","ref":"JSONAPI.View.html#url_for/3"},{"type":"callback","title":"JSONAPI.View.url_for_pagination/3","doc":"","ref":"JSONAPI.View.html#c:url_for_pagination/3"},{"type":"function","title":"JSONAPI.View.url_for_pagination/4","doc":"","ref":"JSONAPI.View.html#url_for_pagination/4"},{"type":"callback","title":"JSONAPI.View.url_for_rel/3","doc":"","ref":"JSONAPI.View.html#c:url_for_rel/3"},{"type":"function","title":"JSONAPI.View.url_for_rel/4","doc":"","ref":"JSONAPI.View.html#url_for_rel/4"},{"type":"callback","title":"JSONAPI.View.visible_fields/2","doc":"","ref":"JSONAPI.View.html#c:visible_fields/2"},{"type":"function","title":"JSONAPI.View.visible_fields/3","doc":"","ref":"JSONAPI.View.html#visible_fields/3"},{"type":"exception","title":"JSONAPI.Exceptions.InvalidQuery","doc":"Defines a generic exception for when an invalid query is received and is unable to be parsed nor handled.\n\nAll JSONAPI exceptions on index routes return a 400.","ref":"JSONAPI.Exceptions.InvalidQuery.html"},{"type":"extras","title":"JSONAPI Elixir","doc":"# JSONAPI Elixir\n\n[![Build](https://github.com/beam-community/jsonapi/actions/workflows/ci.yml/badge.svg)](https://github.com/beam-community/jsonapi/actions/workflows/ci.yml)\n[![Hex.pm version](https://img.shields.io/hexpm/v/jsonapi.svg)](https://hex.pm/packages/jsonapi)\n[![Hex.pm downloads](https://img.shields.io/hexpm/dt/jsonapi.svg)](https://hex.pm/packages/jsonapi)\n[![Hex.pm weekly downloads](https://img.shields.io/hexpm/dw/jsonapi.svg)](https://hex.pm/packages/jsonapi)\n[![Hex.pm daily downloads](https://img.shields.io/hexpm/dd/jsonapi.svg)](https://hex.pm/packages/jsonapi)\n\nA project that will render your data models into [JSONAPI Documents](http://jsonapi.org/format) and parse/verify JSONAPI query strings.","ref":"readme.html"},{"type":"extras","title":"JSONAPI Support - JSONAPI Elixir","doc":"This library implements [version 1.1](https://jsonapi.org/format/1.1/)\nof the JSON:API spec.\n\n- [x] Basic [JSONAPI Document](http://jsonapi.org/format/#document-top-level) encoding\n- [x] Basic support for [compound documents](http://jsonapi.org/format/#document-compound-documents)\n- [x] [Links](http://jsonapi.org/format/#document-links)\n- [x] Relationship links\n- [x] Parsing of `sort` query parameter into Ecto Query order_by\n- [x] Parsing and limiting of `filter` keywords.\n- [x] Handling of [sparse fieldsets](https://jsonapi.org/format/#fetching-sparse-fieldsets)\n- [x] Handling of [includes](https://jsonapi.org/format/#fetching-includes)\n- [x] Handling of [pagination](https://jsonapi.org/format/#fetching-pagination)\n- [x] Handling of top level meta data","ref":"readme.html#jsonapi-support"},{"type":"extras","title":"Documentation - JSONAPI Elixir","doc":"- [Full docs here](https://hexdocs.pm/jsonapi)\n- [JSON API Spec (v1.1)](https://jsonapi.org/format/1.1/)","ref":"readme.html#documentation"},{"type":"extras","title":"Badges - JSONAPI Elixir","doc":"![](https://github.com/jeregrine/jsonapi/workflows/Continuous%20Integration/badge.svg)","ref":"readme.html#badges"},{"type":"extras","title":"How to use with Phoenix - JSONAPI Elixir","doc":"","ref":"readme.html#how-to-use-with-phoenix"},{"type":"extras","title":"Installation - JSONAPI Elixir","doc":"Add the following line to your `mix.deps` file with the desired version to install `jsonapi`.\n\n```elixir\ndefp deps do [\n  ...\n  {:jsonapi, \"~> x.x.x\"}\n  ...\n]\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - JSONAPI Elixir","doc":"Simply add `use JSONAPI.View` either to the top of your view, or to the web.ex view section and add the\nproper functions to your view like so.\n\n```elixir\ndefmodule MyApp.PostView do\n  use JSONAPI.View, type: \"posts\"\n\n  def fields do\n    [:text, :body, :excerpt]\n  end\n\n  def excerpt(post, _conn) do\n    String.slice(post.body, 0..5)\n  end\n\n  def meta(data, _conn) do\n    # this will add meta to each record\n    # To add meta as a top level property, pass as argument to render function (shown below)\n    %{meta_text: \"meta_#{data[:text]}\"}\n  end\n\n  def relationships do\n    # The post's author will be included by default\n    [author: {MyApp.UserView, :include},\n     comments: MyApp.CommentView]\n  end\nend\n```\n\nYou can now call `render(conn, MyApp.PostView, \"show.json\", %{data: my_data, meta: meta})`\nor `\"index.json\"` normally.\n\nIf you'd like to use this without Phoenix simply use the `JSONAPI.View` and call\n`JSONAPI.Serializer.serialize(MyApp.PostView, data, conn, meta)`.","ref":"readme.html#usage"},{"type":"extras","title":"Renaming relationships - JSONAPI Elixir","doc":"If a relationship has a different name in the backend than you would like it to in your API,\nyou can rewrite its name in the `JSONAPI.View`. You pair the view with the name of the relationship\nused in the data (e.g. Ecto schema) to achieve this. Note that you can use a triple instead \nof a pair to add the instruction to always include the relation if desired.\n\n```elixir\ndefmodule MyApp.PostView do\n  use JSONAPI.View, type: \"posts\"\n\n  def relationships do\n    # The `author` will be exposed as `creator` and the `comments` will be \n    # exposed as `critiques` (for some reason).\n    [creator: {:author, MyApp.UserView, :include},\n     critiques: {:comments, MyApp.CommentView}]\n  end\nend\n```","ref":"readme.html#renaming-relationships"},{"type":"extras","title":"Parsing and validating a JSONAPI Request - JSONAPI Elixir","doc":"In your controller you may add\n\n```elixir\nplug JSONAPI.QueryParser,\n  filter: ~w(name),\n  sort: ~w(name title inserted_at),\n  view: PostView\n```\n\nThis will add a `JSONAPI.Config` struct called `jsonapi_query` to your\n`conn.assigns`. If a user tries to sort, filter, include, or requests an\ninvalid fieldset it will raise a `Plug` error that shows the proper error\nmessage.\n\nThe config holds the values parsed into things that are easy to pass into an Ecto\nquery, for example `sort=-name` will be parsed into `sort: [desc: :name]` which\ncan be passed directly to the `order_by` in Ecto.\n\nThis sort of behavior is consistent for includes.\n\nThe `JSONAPI.QueryParser` plug also supports [sparse fieldsets](https://jsonapi.org/format/#fetching-sparse-fieldsets).\nPlease see its documentation for details.","ref":"readme.html#parsing-and-validating-a-jsonapi-request"},{"type":"extras","title":"Camelized or Dasherized Fields - JSONAPI Elixir","doc":"JSONAPI has recommended in the past the use of dashes (`-`) in place of underscore (`_`) as a\nword separator for document member keys. However, as of [JSON API Spec (v1.1)](https://jsonapi.org/format/1.1/), it is now recommended that member names\nare camelCased. This library provides various configuration options for maximum flexibility including serializing outgoing parameters and deserializing incoming parameters.\n\nTransforming fields requires two steps:\n\n1. camelCase _outgoing_ fields requires you to set the `:field_transformation`\n   configuration option. Example:\n\n   ```elixir\n   config :jsonapi,\n     field_transformation: :camelize # or :dasherize, :camelize_shallow, or :dasherize_shallow\n   ```\n\n2. Underscoring _incoming_ params (both query and body) requires you add the\n   `JSONAPI.UnderscoreParameters` Plug to your API's pipeline. This makes it easy to\n   work with changeset data.\n\n   ```elixir\n   pipeline :api do\n     plug JSONAPI.EnsureSpec\n     plug JSONAPI.UnderscoreParameters\n   end\n   ```\n\n3. JSONAPI.Deserializer is a plug designed to make a JSON:API resource object more convenient\n   to work with when creating or updating resources. This plug works by taking the resource\n   object format and flattening it into an easier to manipulate Map.\n\n   Note that the deserializer expects the same casing for your _outgoing_ params as your\n   _incoming_ params.\n\n   Your pipeline in a Phoenix app might look something like this:\n\n   ```elixir\n   pipeline :api do\n     plug JSONAPI.EnsureSpec\n     plug JSONAPI.Deserializer\n     plug JSONAPI.UnderscoreParameters\n   end\n   ```","ref":"readme.html#camelized-or-dasherized-fields"},{"type":"extras","title":"Spec Enforcement - JSONAPI Elixir","doc":"We include a set of Plugs to make enforcing the JSONAPI spec for requests easy. To add spec enforcement to your application, add `JSONAPI.EnsureSpec` to your pipeline:\n\n```elixir\nplug JSONAPI.EnsureSpec\n```\n\nUnder-the-hood `JSONAPI.EnsureSpec` relies on four individual plugs:\n\n- `JSONAPI.ContentTypeNegotiation` — Requires the `Content-Type` and `Accept` headers are set correctly.\n\n- `JSONAPI.FormatRequired` — Verifies that the JSON body matches the expected `%{data: %{attributes: attributes}}` format.\n\n- `JSONAPI.IdRequired` — Confirm the `id` key is present in `%{data: data}` and that it matches the resource's `id` in the URI.\n\n- `JSONAPI.ResponseContentType` — Ensures that you return the correct `Content-Type` header.","ref":"readme.html#spec-enforcement"},{"type":"extras","title":"Configuration - JSONAPI Elixir","doc":"```elixir\nconfig :jsonapi,\n  host: \"www.someotherhost.com\",\n  scheme: \"https\",\n  namespace: \"/api\",\n  field_transformation: :underscore,\n  remove_links: false,\n  json_library: Jason,\n  paginator: nil\n```\n\n- **host**, **scheme**. By default these are pulled from the `conn`, but may be\n  overridden.\n- **namespace**. This optional setting can be used to configure the namespace\n  your resources live at (e.g. given \"http://example.com/api/cars\", `\"/api\"`\n  would be the namespace). See also `JSONAPI.View` for setting on the resource's\n  View itself.\n- **field_transformation**. This option describes how your API's fields word\n  boundaries are marked. [JSON API Spec (v1.1)](https://jsonapi.org/format/1.1/) recommends using camelCase (e.g.\n  `\"favoriteColor\": blue`). If your API uses camelCase fields, set this value to\n  `:camelize`. JSON:API v1.0 recommended using a dash (e.g.\n  `\"favorite-color\": blue`). If your API uses dashed fields, set this value to\n  `:dasherize`. If your API uses underscores (e.g. `\"favorite_color\": \"red\"`)\n  set to `:underscore`. To transform only the top-level field keys,  use\n  `:camelize_shallow` or `:dasherize_shallow`.\n- **remove_links**. `links` data can optionally be removed from the payload via\n  setting the configuration above to `true`. Defaults to `false`.\n- **json_library**. Defaults to [Jason](https://hex.pm/packages/jason).\n- **paginator**. Module implementing pagination links generation. Defaults to `nil`.","ref":"readme.html#configuration"},{"type":"extras","title":"Pagination - JSONAPI Elixir","doc":"Pagination links can be generated by overriding the `JSONAPI.View.pagination_links/4` callback of your view and returning a map containing the links.\n\n```elixir\n...\n\ndef pagination_links(data, conn, page, options) do\n  %{first: nil, last: nil, prev: nil, next: nil}\nend\n...\n```\n\nAlternatively you can define generic pagination strategies by implementing a module\nconforming to the `JSONAPI.Paginator` behavior\n\n```elixir\ndefmodule PageBasedPaginator do\n  @moduledoc \"\"\"\n  Page based pagination strategy\n  \"\"\"\n\n  @behaviour JSONAPI.Paginator\n\n  @impl true\n  def paginate(data, view, conn, page, options) do\n    number =\n      page\n      |> Map.get(\"page\", \"0\")\n      |> String.to_integer()\n\n    size =\n      page\n      |> Map.get(\"size\", \"0\")\n      |> String.to_integer()\n\n    total_pages = Keyword.get(options, :total_pages, 0)\n\n    %{\n      first: view.url_for_pagination(data, conn, Map.put(page, \"page\", \"1\")),\n      last: view.url_for_pagination(data, conn, Map.put(page, \"page\", total_pages)),\n      next: next_link(data, view, conn, number, size, total_pages),\n      prev: previous_link(data, view, conn, number, size)\n    }\n  end\n\n  defp next_link(data, view, conn, page, size, total_pages)\n       when page   1,\n       do: view.url_for_pagination(data, conn, %{size: size, page: page - 1})\n\n  defp previous_link(_data, _view, _conn, _page, _size),\n    do: nil\nend\n```\n\nand configuring it as the global pagination logic in your `mix.config`\n\n```elixir\nconfig :jsonapi, :paginator, PageBasedPaginator\n```\n\nor as the view pagination logic when using `JSONAPI.View`\n\n```elixir\nuse JSONAPI.View, paginator: PageBasedPaginator\n```\n\nLinks can be generated using the `JSONAPI.Config.page` information stored in the connection assign `jsonapi_query` and by passing additional information to the `pagination_links/4` callback or your paginator module by passing `options` from your controller.\n\nActual pagination is expected to be handled in your application logic and is outside the scope of this library.","ref":"readme.html#pagination"},{"type":"extras","title":"Other - JSONAPI Elixir","doc":"- Feel free to make PR's. I will do my best to respond within a day or two.\n- If you want to take one of the TODO items just create an issue or PR and let me know so we avoid duplication.\n- If you need help, I am on irc and twitter.\n- [Example project](https://github.com/alexjp/jsonapi-testing)","ref":"readme.html#other"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.0"}}